from lesscli import add_subcommand
from arms.utils.wordstyle import word_to_style, WordStyle
from arms.utils.grammar import parse_json5, guess_java_type, judge_class_name, is_column_sql, parse_column_sql, \
    sql_type_guess_java_type
from arms.utils.templates import render_java_vo_body, render_java_vo_frame, render_java_postkind_interface, \
    render_java_pagekind_interface, render_java_getkind_interface, render_java_controller_frame, \
    render_java_entity_body, render_java_entity_frame, \
    render_java_service_impl_body, render_java_service_body, render_java_mapper_body, make_interface_name
from InquirerPy import inquirer
import json
import os
from pathlib import Path
import pyperclip
import re
import requests
from rich.console import Console
from arms.utils.wordstyle import word_to_style

console = Console()


def blank_func():
    pass


def alert_and_exit(text):
    console.print(text.replace('[', '\\['), style="bold red")
    exit(1)


def makedir(dir_path):
    Path(dir_path).mkdir(parents=True, exist_ok=True)


def get_mockjs_item(data, key):
    for k in data.keys():
        if k == key or k.startswith(f'{key}|'):
            return data[k]
    return data[key]  # 抛出KeyError


def find_dto_data(obj):
    data = get_mockjs_item(obj, 'data')
    if 'totalCount' in data and 'currentPage' in data and 'numPerPage' in data:
        return get_mockjs_item(data, 'recordList')[0]
    elif isinstance(data, list):
        return data[0]
    else:
        return data


def parse_json5_to_dll(json5_content, is_dto: bool, is_put: bool):
    """
    输入json5文本，输出：[(var_name, java_type, comment, required)]
    如果is_dto为True，会智能地找到关键data
    """
    data, comment_dict = parse_json5(json5_content)
    if is_dto:
        data = find_dto_data(data)
    schema_list = []
    for var_name, mockjs in data.items():
        java_type = guess_java_type(mockjs)
        comment = comment_dict.get(var_name, '').replace('"', "'")
        if is_put:
            required = comment.startswith('必传')
        else:
            required = not comment.startswith('非必传')
        if '|' in var_name:
            var_name = var_name.split('|', 1)[0]
        schema_list.append((var_name, java_type, comment, required))
    return schema_list


def path_to_package(path):
    return path.split('src/main/java/', 1)[-1].rstrip('/').replace('/', '.')


def make_vo_content(vo_path, vo_class, title, json5_content, is_dto, is_put):
    vo_package = path_to_package(vo_path)
    schema_list = parse_json5_to_dll(json5_content, is_dto, is_put)
    vo_body = render_java_vo_body(schema_list)
    if is_dto:
        title += '的返回对象'
    else:
        title += '的请求对象'
    return render_java_vo_frame(vo_package, vo_class, title, vo_body)


def codegen_controller():
    """
    创建或修改java controller源代码
    """
    # 1.配置目录（与package）
    if not os.path.isdir('.git'):
        alert_and_exit('Please change workdir to top! or run "git init" first.')
    config_path = '.codegen'
    try:
        config_data = json.loads(open(config_path).read())
    except:
        config_data = {}
    default_controller_dir = config_data.get('CONTROLLER_DIR', '')
    controller_dir = inquirer.text(message="请输入controller目录:", default=default_controller_dir).execute()
    assert '/' in controller_dir, 'controller目录至少应是多级目录'
    controller_path = Path() / controller_dir
    assert controller_path.is_dir(), 'controller目录不存在，请先创建目录'
    if default_controller_dir != controller_dir:
        config_data['CONTROLLER_DIR'] = controller_dir
        vo_dir = inquirer.text(message="请输入Vo（Value Object请求对象）目录:").execute()
        vo_path = Path() / vo_dir
        assert vo_path.is_dir(), 'VO目录不存在，请先创建目录'
        config_data['VO_DIR'] = vo_dir
        dto_dir = inquirer.text(message="请输入Dto（返回对象）目录:").execute()
        dto_path = Path() / dto_dir
        assert dto_path.is_dir(), 'DTO目录不存在，请先创建目录'
        config_data['DTO_DIR'] = dto_dir
    package_name = path_to_package(controller_dir)
    # 2.输入类名
    controller_class = inquirer.text(message="请输入controller类名:").execute()
    judge_class_name(controller_class, 'Controller')
    controller_file = controller_path / f'{controller_class}.java'
    # 3.输入yapi链接和token
    default_yapi_token = config_data.get('YAPI_TOKEN', '')
    yapi_token = inquirer.text(message="请输入YAPI项目TOKEN:", default=default_yapi_token).execute()
    config_data['YAPI_TOKEN'] = yapi_token
    yapi_url = inquirer.text(message="请输入YAPI页面链接:").execute()
    match_ret = re.findall('^(https?://[^/]+)/.*/interface/api/([0-9]+)$', yapi_url)
    assert match_ret, '该链接不是正确的YAPI接口页面链接'
    yapi_host, interface_id = match_ret[0]
    resp = requests.get(f'{yapi_host}/api/interface/get', params={'id': interface_id, 'token': yapi_token})
    assert resp.status_code == 200, f'YAPI接口返回{resp.status_code}'
    yapi_data = resp.json()
    assert yapi_data['errcode'] == 0, f'YAPI接口返回errmsg:{yapi_data["errmsg"]}'
    # 4.选择controller子目录
    subpath_dict = {}  # dict[path.name, path]
    for subpath in controller_path.glob('*'):
        if subpath.is_dir():
            subpath_dict[subpath.name] = subpath
    if subpath_dict:
        subpath_dict['.'] = controller_path
        subpath_name = inquirer.select(
            message='请选择controller子目录:',
            choices=list(subpath_dict.keys()),
            default=None,
        ).execute()
        controller_file = subpath_dict[subpath_name] / f'{controller_class}.java'
        package_name += f'.{subpath_name}'
    else:
        subpath_name = '.'
    # 5.收集数据，准备创建controller
    yapi_data = yapi_data['data']
    method = yapi_data['method'].lower()
    path = yapi_data['path'] if subpath_name == '.' else f'/{subpath_name}' + yapi_data['path']
    title = yapi_data['title']
    req_query = yapi_data.get('req_params', []) + yapi_data.get('req_query', [])
    vo_json5 = yapi_data.get('req_body_other', '')
    dto_json5 = yapi_data.get('res_body', '')
    if method in ['post', 'put']:
        assert yapi_data.get('req_body_type') == 'json' and vo_json5, 'YAPI接口请求对象不能为空'
    assert yapi_data.get('res_body_type') == 'json' and dto_json5, 'YAPI接口返回对象不能为空'
    vo_class = dto_class = ''
    vo_class_prefix = word_to_style(make_interface_name(method, path), WordStyle.upper_camel)
    if vo_json5:
        vo_class = inquirer.text(message="请输入Vo（Value Object请求对象）类名（可为空）:", default=f'{vo_class_prefix}Vo').execute()
        if vo_class:
            judge_class_name(vo_class, 'Vo')
            vo_content = make_vo_content(config_data['VO_DIR'], vo_class, title, vo_json5, is_dto=False,
                                         is_put=(method == 'put'))
            vo_file = Path(config_data['VO_DIR']) / f'{vo_class}.java'
            with vo_file.open('w') as f:
                f.write(vo_content)
        else:
            vo_class = 'Object'
    if dto_json5:
        dto_class = inquirer.text(message="请输入Dto（返回对象）类名（可为空）:", default=f'{vo_class_prefix}Dto').execute()
        if dto_class:
            judge_class_name(dto_class, 'Dto')
            dto_content = make_vo_content(config_data['DTO_DIR'], dto_class, title, dto_json5, is_dto=True, is_put=True)
            dto_file = Path(config_data['DTO_DIR']) / f'{dto_class}.java'
            with dto_file.open('w') as f:
                f.write(dto_content)
        else:
            dto_class = 'Object'
    if method in ['post', 'put']:
        interface_content = render_java_postkind_interface(method, path, title, vo_class, dto_class, req_query)
    elif 'pageNum' in str(req_query) and 'numPerPage' in str(req_query):
        interface_content = render_java_pagekind_interface(method, path, title, dto_class, req_query)
    else:
        interface_content = render_java_getkind_interface(method, path, title, dto_class, req_query)
    if not controller_file.exists():
        controller_content = render_java_controller_frame(package_name, controller_class, interface_content)
    else:
        controller_content = controller_file.open().read().strip().rstrip('}') + interface_content + '\n}\n'
    with controller_file.open('w') as f:
        f.write(controller_content)
    with open(config_path, 'w') as f:
        f.write(json.dumps(config_data))


def codegen_entity():
    """
    基于建表SQL创建java entity源代码
    如果文件已存在则覆盖
    """
    # 1.配置目录（与package）
    if not os.path.isdir('.git'):
        alert_and_exit('Please change workdir to top! or run "git init" first.')
    config_path = '.codegen'
    try:
        config_data = json.loads(open(config_path).read())
    except:
        config_data = {}
    default_entity_dir = config_data.get('ENTITY_DIR', '')
    config_data['ENTITY_DIR'] = entity_dir = inquirer.text(message="请输入entity目录:", default=default_entity_dir).execute()
    assert '/' in entity_dir, 'entity目录至少应是多级目录'
    entity_path = Path() / entity_dir
    assert entity_path.is_dir(), 'entity目录不存在，请先创建目录'
    package_name = path_to_package(entity_dir)
    entity_class = inquirer.text(message="请输入Entity实体类名:").execute()
    judge_class_name(entity_class, '')
    entity_comment = inquirer.text(message="请输入实体的中文描述:").execute()
    assert entity_comment, '实体的中文描述不能为空'
    sql_content = inquirer.text(message="请输入建表SQL(ctrl+o完成输入):", multiline=True).execute()
    seed = re.compile(r'CREATE TABLE *`?(\w+)`?\s*\((.*)\)', flags=re.I | re.DOTALL)
    assert sql_content.strip(), '建表SQL不能为空'
    match_size = len(find_ret := seed.findall(sql_content))
    assert match_size == 1, '无法解析该建表SQL'
    table_name, sql_body = find_ret[0]
    schema_list = []  # [(col_name, var_name, java_type, comment)]
    for sql_line in sql_body.splitlines():
        if sql_line and is_column_sql(sql_line):
            col_name, sql_type, comment = parse_column_sql(sql_line)
            var_name = word_to_style(col_name, WordStyle.lower_camel)
            java_type = sql_type_guess_java_type(sql_type, comment)
            schema_list.append((col_name, var_name, java_type, comment))
    entity_body = render_java_entity_body(schema_list)
    entity_content = render_java_entity_frame(package_name, entity_class, entity_comment, table_name, entity_body)
    dto_file = entity_path / f'{entity_class}.java'
    with dto_file.open('w') as f:
        f.write(entity_content)
    with open(config_path, 'w') as f:
        f.write(json.dumps(config_data))


def codegen_service():
    """
    创建java service源代码
    如果文件已存在则覆盖
    """
    # 1.配置目录（与package）
    if not os.path.isdir('.git'):
        alert_and_exit('Please change workdir to top! or run "git init" first.')
    config_path = '.codegen'
    try:
        config_data = json.loads(open(config_path).read())
    except:
        config_data = {}
    default_service_dir = config_data.get('SERVICE_DIR', '')
    config_data['SERVICE_DIR'] = service_dir = inquirer.text(message="请输入service目录:",
                                                             default=default_service_dir).execute()
    assert '/' in service_dir, 'service目录至少应是多级目录'
    service_path = Path() / service_dir
    assert service_path.is_dir(), 'service目录不存在，请先创建目录'
    service_package = path_to_package(service_dir)
    # 2.mapper目录
    default_mapper_dir = config_data.get('MAPPER_DIR', '')
    config_data['MAPPER_DIR'] = mapper_dir = inquirer.text(message="请输入mapper目录:", default=default_mapper_dir).execute()
    assert '/' in mapper_dir, 'mapper目录至少应是多级目录'
    mapper_path = Path() / mapper_dir
    assert mapper_path.is_dir(), 'mapper目录不存在，请先创建目录'
    mapper_package = path_to_package(mapper_dir)
    # 3.实体目录
    default_entity_dir = config_data.get('ENTITY_DIR', '')
    config_data['ENTITY_DIR'] = entity_dir = inquirer.text(message="请输入entity目录:", default=default_entity_dir).execute()
    assert '/' in entity_dir, 'entity目录至少应是多级目录'
    entity_path = Path() / entity_dir
    assert entity_path.is_dir(), 'entity目录不存在，请先创建目录'
    entity_package = path_to_package(entity_dir)
    entity_class = inquirer.text(message="请输入Entity实体类名:").execute()
    judge_class_name(entity_class, '')
    # 4.创建各个文件(mapper->service->serviceimpl)
    makedir(service_dir.rstrip('/') + '/impl')
    mapper_content = render_java_mapper_body(mapper_package, entity_package, class_name=entity_class)
    mapper_file = mapper_path / f'{entity_class}Mapper.java'
    with mapper_file.open('w') as f:
        f.write(mapper_content)
    service_content = render_java_service_body(service_package, entity_package, class_name=entity_class)
    service_file = service_path / f'{entity_class}Service.java'
    with service_file.open('w') as f:
        f.write(service_content)
    service_impl_content = render_java_service_impl_body(service_package, entity_package, mapper_package,
                                                         class_name=entity_class)
    service_impl_file = service_path / 'impl' / f'{entity_class}ServiceImpl.java'
    with service_impl_file.open('w') as f:
        f.write(service_impl_content)
    with open(config_path, 'w') as f:
        f.write(json.dumps(config_data))


@add_subcommand('controller', codegen_controller)
@add_subcommand('entity', codegen_entity)
@add_subcommand('service', codegen_service)
def run_codegen():
    """
    生成JAVA文件或修改文件
    自动识别编程语言
    """
    pass



from typing import Tuple, List, NamedTuple, Dict
from enum import Enum
from arms.utils.common import captitalize
import re


class WordStyle(Enum):
    upper_camel = 1
    lower_camel = 2
    upper_snake = 3
    lower_snake = 4
    upper_minus = 5
    lower_minus = 6
    other = 7


class WordSeed(NamedTuple):
    word_style: WordStyle
    tokens: List[str]

    def __str__(self) -> str:
        if not self.tokens:
            return ''
        elif self.word_style == WordStyle.upper_camel:
            return ''.join(token.capitalize() for token in self.tokens)
        elif self.word_style == WordStyle.lower_camel:
            return self.tokens[0] + ''.join(token.capitalize() for token in self.tokens[1:])
        elif self.word_style == WordStyle.upper_snake:
            return '_'.join(self.tokens).upper()
        elif self.word_style == WordStyle.lower_snake:
            return '_'.join(self.tokens)
        elif self.word_style == WordStyle.upper_minus:
            return '-'.join(self.tokens).upper()
        elif self.word_style == WordStyle.lower_minus:
            return '-'.join(self.tokens)
        else:
            return ''.join(self.tokens)

    @classmethod
    def of(cls, word: str) -> 'WordSeed':
        if '_' in word:
            if re.match(r'^[a-z0-9_]+$', word):
                return WordSeed(WordStyle.lower_snake, word.split('_'))
            elif re.match(r'^[A-Z0-9_]+$', word):
                return WordSeed(WordStyle.upper_snake, [seg.lower() for seg in word.split('_')])
        elif '-' in word:
            if re.match(r'^[a-z0-9-]+$', word):
                return WordSeed(WordStyle.lower_minus, word.split('-'))
            elif re.match(r'^[A-Z0-9-]+$', word):
                return WordSeed(WordStyle.upper_minus, [seg.lower() for seg in word.split('-')])
        elif re.match(r'^[A-Z][A-Za-z0-9]+$', word):
            return WordSeed(WordStyle.upper_camel, [seg.lower() for seg in re.findall(r'[A-Z][a-z0-9]*', word)])
        elif re.match(r'^[a-z][A-Za-z0-9]+$', word):
            return WordSeed(WordStyle.lower_camel, [seg.lower() for seg in re.findall(r'[A-Z][a-z0-9]*', captitalize(word))])
        return WordSeed(WordStyle.other, [word])


def word_to_style(word: str, style: WordStyle) -> str:
    word_seed = WordSeed.of(word)
    return str(WordSeed(style, word_seed.tokens))


def replace_dict(oldword: str, newword: str):
    old_word_seed = WordSeed.of(oldword)
    new_word_seed = WordSeed.of(newword)
    if len(old_word_seed.tokens) == 1:  # 如果oldword非词组，则newword也按非词组处理
        new_word_seed = WordSeed.of(''.join(new_word_seed.tokens))
    return {
        str(WordSeed(word_style, old_word_seed.tokens)): str(WordSeed(word_style, new_word_seed.tokens))
        for word_style
        in [
            WordStyle.upper_snake, WordStyle.lower_snake,
            WordStyle.upper_minus, WordStyle.lower_minus,
            WordStyle.upper_camel, WordStyle.lower_camel,
        ]
    }


def replace_all(text: str, repl_dict: Dict):
    for in_word, out_word in repl_dict.items():
        text = text.replace(in_word, out_word)
    return text



from lesscli import add_subcommand
from arms.utils.grammar import is_column_sql, parse_column_sql, guess_mockjs
from arms.utils.templates import render_sql_line, render_create_sql_frame, render_json5, \
    render_yapi_query, render_interface_query_body
from arms.utils.wordstyle import word_to_style, WordStyle
from InquirerPy import inquirer
import pyperclip
import re
from rich.console import Console
from rich.syntax import Syntax


console = Console()


def copy_board():
    """
    展示当前的粘贴板内容（不超过1000行和10KB），
    选择确认/放弃
    选确认则返回粘贴板内容
    """
    splitor_start = '─────────────────────────'
    splitor_end = '───────────────────────────'
    content = pyperclip.paste()
    if len(content.splitlines()) > 1000 or len(content) > 10240:
        content = '（粘贴板内容超过了1000行或10KB）'
    content = content.strip()
    answer = inquirer.select(
        message=splitor_start + '\n' + content + '\n' + splitor_end,
        choices=['确认使用粘贴板内容', '放弃'],
        default=None,
    ).execute()
    if answer == '放弃':
        exit(0)
    return content


def design_shimo_to_sql():
    """
    复制石墨表格，转换成建表SQL，保存到粘贴板
    石墨表格的格式：字段名(驼峰)、类型(SQL)、注释
    """
    shimo_content = copy_board()
    camel_stype = inquirer.select(
        message="请选择数据库字段风格",
        choices=['蛇形', '驼峰'],
        default=None,
    ).execute() == '驼峰'
    table_name = inquirer.text(message="请输入数据库表名:").execute()
    table_comment = inquirer.text(message="请输入表的中文注释:").execute()
    sql_lines = []
    primary_key = ''
    for pos, row in enumerate(shimo_content.splitlines()):
        seg_size = len(segs := row.split('\t'))
        assert seg_size == 3, f'第{pos+1}行包含{seg_size}列，应该包含3列'
        name, sql_type, comment = segs
        assert sql_type, f'第{pos + 1}行的第2列(sql_type)不能为空'
        if not camel_stype:
            col_name = word_to_style(name, WordStyle.lower_snake)
        else:
            col_name = name
        if pos == 0:
            primary_key = col_name
        mockjs = guess_mockjs(sql_type, name, comment)
        sql_lines.append(render_sql_line(col_name, sql_type, comment, mockjs, pos == 0))
    sql_content = render_create_sql_frame(table_name, table_comment, primary_key, sql_lines)
    print('创建数据库的SQL如下：')
    console.print(Syntax(sql_content, 'sql'))
    pyperclip.copy(sql_content)
    print('已复制到粘贴板！')


def design_sql_to_shimo():
    """
    复制建表SQL，转换成石墨表格，保存到粘贴板
    石墨表格的格式：字段名(驼峰)、类型(SQL)、注释
    """
    sql_content = copy_board()
    seed = re.compile(r'CREATE TABLE *`?(\w+)`? *\((.*)\)', flags=re.I|re.DOTALL)
    match_size = len(find_ret := seed.findall(sql_content))
    assert match_size == 1, '无法解析该建表SQL'
    table_name, sql_body = find_ret[0]
    shimo_lines = []
    for sql_line in sql_body.splitlines():
        if sql_line and is_column_sql(sql_line):
            col_name, sql_type, comment = parse_column_sql(sql_line)
            name = word_to_style(col_name, WordStyle.lower_camel)
            shimo_line = '\t'.join([name, sql_type, comment])
            shimo_lines.append(shimo_line)
    shimo_content = '\n'.join(shimo_lines)
    print('定义实体的石墨文本如下：')
    console.print(Syntax(shimo_content, 'text'))
    pyperclip.copy(shimo_content)
    print('已复制到粘贴板！')


def design_shimo_to_json5():
    """
    复制石墨表格，转化为json5，保存到粘贴板
    石墨表格的格式：字段名(驼峰)、类型(SQL)、注释
    """
    shimo_content = copy_board()
    shimo_rows = []
    for pos, row in enumerate(shimo_content.splitlines()):
        seg_size = len(segs := row.split('\t'))
        assert seg_size == 3, f'第{pos + 1}行包含{seg_size}列，应该包含3列'
        name, sql_type, comment = segs
        assert sql_type, f'第{pos + 1}行的第2列(sql_type)不能为空'
        mockjs = guess_mockjs(sql_type, name, comment)
        shimo_rows.append((name, sql_type, mockjs, comment))
    tpl_type = inquirer.select(
        message='请选择结果模版:',
        choices=['保持原状', '返回普通对象模版', '返回数组对象(无分页)模版', '分页返回对象模版'],
        default=None,
    ).execute()
    json5_content = render_json5(shimo_rows, tpl_type)
    print('json5内容如下：')
    console.print(Syntax(json5_content, 'json'))
    pyperclip.copy(json5_content)
    print('已复制到粘贴板！')


def design_shimo_to_query():
    """
    复制石墨表格，转化为YAPI文本或JAVA代码，保存到粘贴板
    石墨表格的格式：字段名(驼峰)、类型(SQL)、注释
    YAPI批量添加Query文本的格式：name:example（只能把注释放到example的位置）
    """
    shimo_content = copy_board()
    shimo_rows = []
    for pos, row in enumerate(shimo_content.splitlines()):
        seg_size = len(segs := row.split('\t'))
        assert seg_size == 3, f'第{pos + 1}行包含{seg_size}列，应该包含3列'
        shimo_rows.append(segs)
    answer = inquirer.select(
        message='请选择分页请求参数：',
        choices=['pageNum & numPerPage', '不需分页'],
        default=None,
    ).execute()
    lang_type = inquirer.select(
        message='请选择输出格式：',
        choices=['YAPI批量添加Query', 'JAVA请求参数'],
        default=None,
    ).execute()
    if answer == 'pageNum & numPerPage':
        yapi_query_content = 'pageNum:当前页码，从1开始\nnumPerPage:分页大小\n'
        java_query_content = ',\n        @Valid @ParameterObject PageReq pageReq'
    else:
        yapi_query_content = ''
        java_query_content = ''
    if 'YAPI' in lang_type:
        yapi_query_content += render_yapi_query(shimo_rows)
        print('YAPI批量添加Query文本如下：')
        console.print(Syntax(yapi_query_content, 'text'))
        pyperclip.copy(yapi_query_content)
    else:
        req_query = [{'name': name, 'required': '0', 'desc': comment} for name, sql_type, comment in shimo_rows]
        java_query_content = render_interface_query_body('', req_query) + java_query_content
        print('JAVA请求参数的代码如下：')
        console.print(Syntax(java_query_content, 'java'))
        pyperclip.copy(java_query_content)
    print('已复制到粘贴板！')


def design_sql_join():
    """
    复制多个建表SQL，转换成联合查询SQL，保存到粘贴板
    需要select的字段前加上前缀「out_name:」，相同out_name的默认会join
    """
    sql_content = pyperclip.paste()
    seed = re.compile(r'CREATE TABLE *`?(\w+)`? *\((.*?)\) ENGINE=InnoDB', flags=re.I|re.DOTALL)
    find_ret = seed.findall(sql_content)
    assert find_ret, f'无法解析粘贴板的建表SQL：{repr(sql_content[:100])}...'
    col_index = {}  # dict[col_name: table_name]
    select_list = []
    where_table = ''
    join_list = []
    for table_name, sql_body in find_ret:
        if not where_table:
            where_table = table_name
        for sql_line in sql_body.splitlines():
            if sql_line and is_column_sql(sql_line):
                col_name, sql_type, comment = parse_column_sql(sql_line)
                if col_name.count('`') == 1:
                    col_name += '`'
                elif col_name.endswith(':'):
                    col_name += sql_type
                out_name = ''
                if col_name.startswith(':'):
                    out_name = col_name = col_name.lstrip(':')
                    select_list.append(f'{table_name}.{col_name}')
                elif ':' in col_name:
                    out_name, col_name = col_name.split(':', 1)
                    out_name = f'`{out_name}`'
                    select_list.append(f'{table_name}.{col_name} AS {out_name}')
                if out_name:
                    if out_name in col_index:
                        left_table_name, left_col_name = col_index[out_name]
                        join_list.append(f'LEFT JOIN {table_name} ON {table_name}.{col_name}={left_table_name}.{left_col_name}')
                    else:
                        col_index[out_name] = table_name, col_name
    assert select_list, '提示：需要select的字段前加上前缀「out_name:」或「:」，相同out_name的默认会join'
    sql_result = 'SELECT '
    sql_result += ','.join(select_list) + '\n'
    sql_result += f'FROM {where_table} \n'
    sql_result += ' \n'.join(join_list)
    console.print(Syntax(sql_result, 'sql'))
    pyperclip.copy(sql_result)
    print('已复制到粘贴板！')


def design_sql_to_er_diagram():
    """
    复制多个建表SQL，转换成E-R图的markdown文本，保存到粘贴板
    需要重点关注sql的外键设置和Not NULL设置
    """
    all_entity: dict = {}  # Dict[table_name] -> entity: List[Dict[name|type|nullable] -> str|bool]
    all_column: dict = {}  # Dict[table_name, col_name] -> Dict[name|type|nullable] -> str|bool
    many_to_one: dict = {}  # Dict[from_table, to_table] -> nullable: bool
    sql_content = pyperclip.paste()
    seed = re.compile(r'CREATE TABLE *`?(\w+)`? *\((.*?)\) ENGINE=InnoDB', flags=re.I | re.DOTALL)
    find_ret = seed.findall(sql_content)
    assert find_ret, f'无法解析粘贴板的建表SQL：{repr(sql_content[:100])}...'
    for table_name, sql_body in find_ret:
        all_entity[table_name] = []
        for sql_line in sql_body.splitlines():
            if sql_line and is_column_sql(sql_line):
                col_name, sql_type, _ = parse_column_sql(sql_line)
                sql_type = re.findall(r'^\w+', sql_type)[0]
                nullable = 'NOT NULL' not in sql_line.upper()
                column_dict = {'name': col_name, 'type': sql_type, 'nullable': nullable}
                all_entity[table_name].append(column_dict)
                all_column[table_name, col_name] = column_dict
            elif sql_line.strip().upper().startswith('UNIQUE '):
                seed_uk = re.compile(r'UNIQUE KEY *.*\((.*?)\)')
                col_name_part = seed_uk.findall(sql_line)[0]
                col_names = col_name_part.replace('`', '').replace(',', ' ').split()
                for pos, col_name in enumerate(col_names):
                    assert (table_name, col_name) in all_column, f'需补充唯一键索引字段定义：{table_name}.{col_name}'
                    if len(col_names) == 1:
                        all_column[table_name, col_name]['type'] += '_UK'
                    else:
                        all_column[table_name, col_name]['type'] += f'_UK{pos}'
            elif sql_line.strip().upper().startswith(('CONSTRAINT ', 'FOREIGN ')):
                seed_fk = re.compile(r'FOREIGN KEY *\((.*?)\) *REFERENCES *`?(\w+)`? *\((.*?)\)', flags=re.I | re.DOTALL)
                cur_col, foreign_table, foreign_col = seed_fk.findall(sql_line)[0]
                cur_col = cur_col.strip('`')
                assert (table_name, cur_col) in all_column, f'需补充外键字段定义：{table_name}.{cur_col}'
                many_to_one[table_name, foreign_table] = all_column[table_name, cur_col]['nullable']
                all_column[table_name, cur_col]['type'] += '_FK'
    md_results = ['```mermaid', 'erDiagram']
    for table_name, column_list in all_entity.items():
        md_results.append(table_name + ' {')
        for column_dict in column_list:
            if column_dict["name"].lower().endswith('id') or '_' in column_dict["type"] \
                    or 'json' in column_dict["type"].lower() or 'enum' in column_dict["type"].lower():
                md_results.append(f'    {column_dict["type"]} {column_dict["name"]}')
        md_results.append('}')
    for (cur_name, foreign_table), nullable in many_to_one.items():
        if nullable:
            md_results.append(f'{cur_name} }}o--o| {foreign_table} : ""')
        else:
            md_results.append(f'{cur_name} }}o--|| {foreign_table} : ""')
    md_results.append('```')
    md_result = '\n'.join(md_results)
    console.print(Syntax(md_result, 'markdown'))
    pyperclip.copy(md_result)
    print('已复制到粘贴板！')


@add_subcommand('shimo-to-sql', design_shimo_to_sql)
@add_subcommand('sql-to-shimo', design_sql_to_shimo)
@add_subcommand('shimo-to-json5', design_shimo_to_json5)
@add_subcommand('shimo-to-query', design_shimo_to_query)
@add_subcommand('sql-join', design_sql_join)
@add_subcommand('sql-to-er', design_sql_to_er_diagram)
def run_design():
    """生成文本并复制"""
    pass



import ast
import json
import re
from arms.utils.wordstyle import word_to_style, WordStyle


def render_create_sql_frame(table_name: str, table_comment: str, primary_key: str, sql_lines: list):
    sql_body = '\n'.join(sql_lines)
    return f"""CREATE TABLE `{table_name}` (
{sql_body}
    PRIMARY KEY (`{primary_key}`)
) ENGINE=InnoDB DEFAULT CHARSET=utf8 COMMENT='{table_comment}';"""


def render_sql_line(col_name: str, sql_type: str, comment: str, mockjs: str, is_primary: bool):
    if mockjs.startswith('@natural'):
        sql_type = 'bigint(20) unsigned'
    elif sql_type.lower() == 'int':
        sql_type += '(11)'
    elif sql_type.lower() == 'bigint':
        sql_type += '(20) unsigned'
    elif sql_type.lower() == 'tinyint':
        sql_type += '(4)'
    elif sql_type.lower() == 'varchar':
        sql_type += '(200)'
    if col_name.lower().replace('_','') == 'nickname':
        sql_type += ' COLLATE utf8mb4_unicode_ci'
    comment = comment.replace("'", '"')
    default_part = ' DEFAULT NULL'
    if is_primary:
        default_part = ' NOT NULL'
    elif sql_type.lower() in ['text', 'blob']:
        default_part = ''
    elif col_name.lower().replace('_','') in ['createdat', 'updatedat', 'createtime', 'updatetime']:
        default_part = ' DEFAULT CURRENT_TIMESTAMP'
    extra_part = ''
    if col_name.lower().replace('_','') in ['updatedat', 'updatetime']:
        extra_part = ' ON UPDATE CURRENT_TIMESTAMP'
    elif is_primary and 'int' in sql_type.lower():
        extra_part = ' AUTO_INCREMENT'
    return f"""    `{col_name}` {sql_type}{default_part}{extra_part} COMMENT '{comment}',"""


def quote_mockjs(mockjs: str):
    if re.match('^[0-9.]+$', mockjs) or mockjs in ['true', 'false']:
        return str(mockjs)
    if mockjs.startswith(('"', "'")) and mockjs.endswith(('"', "'")):
        mockjs = ast.literal_eval(mockjs)
    return json.dumps(mockjs, ensure_ascii=False)


def render_json5(shimo_rows, tpl_type):
    indent = '    '
    if tpl_type == '返回普通对象模版':
        start_lines = [
            '{',
            f'{indent}"code": 0,',
            f'{indent}"msg": "成功",',
            f'{indent}"data": {{',
        ]
        end_lines = [f'{indent}}}', '}']
        depth = 2
    elif tpl_type == '返回数组对象(无分页)模版':
        start_lines = [
            '{',
            f'{indent}"code": 0,',
            f'{indent}"msg": "成功",',
            f'{indent}"data|3": [',
            f'{indent * 2}{{',
        ]
        end_lines = [
            f'{indent * 2}}}',
            f'{indent}]',
            '}',
        ]
        depth = 3
    elif tpl_type == '分页返回对象模版':
        start_lines = [
            '{',
            f'{indent}"code": 0,',
            f'{indent}"msg": "成功",',
            f'{indent}"data": {{',
            f'{indent * 2}"totalCount": 10, //总记录数',
            f'{indent * 2}"currentPage": 1, //当前页码',
            f'{indent * 2}"numPerPage": 10, //分页大小',
            f'{indent * 2}"recordList|10": [',
            f'{indent * 3}{{',
        ]
        end_lines = [
            f'{indent * 3}}}',
            f'{indent * 2}]',
            f'{indent}}}',
            '}'
        ]
        depth = 4
    else:
        start_lines = ['{']
        end_lines = ['}']
        depth = 1
    body_lines = []
    for name, sql_type, mockjs, comment in shimo_rows:
        mockjs = quote_mockjs(mockjs)
        comment = comment.replace('//', ' ').replace('\n', ' ')
        line = f'''{indent * depth}"{name}": {mockjs}, //{comment}'''
        body_lines.append(line)
    return '\n'.join(start_lines + body_lines + end_lines)


def render_yapi_query(shimo_rows):
    body_lines = []
    for name, sql_type, comment in shimo_rows:
        comment = comment.replace(':', '=').replace('\n', ';')
        line = f'''{name}:{comment}'''
        body_lines.append(line)
    return '\n'.join(body_lines)


def render_java_controller_frame(package_name, class_name, interface_body):
    code_body = f"""package {package_name};

import io.swagger.v3.oas.annotations.Operation;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import javax.validation.Valid;

@Validated
@RestController
public class {class_name} {{
{interface_body}
}}"""
    return code_body


def render_interface_query_body(path, req_query: list):
    if not req_query:
        return ''
    indent = '        '
    query_lines = []
    for query_item in req_query:
        name, required, desc = query_item['name'], query_item.get('required'), query_item.get('desc') or query_item.get('example', '')
        if name in ['pageNum', 'numPerPage']:
            continue
        desc = desc.replace('"', "'")
        if '{' + name + '}' in path:
            query_lines.append(f"""{indent}@Parameter(description = "{desc}") @PathVariable Long {name}""")
        elif required == '0':
            query_lines.append(f"""{indent}@Parameter(description = "{desc}") @RequestParam(required = false) String {name}""")
        else:
            query_lines.append(f"""{indent}@Parameter(description = "{desc}") @RequestParam String {name}""")
    query_body = '\n' + ',\n'.join(query_lines)
    return query_body


def make_interface_name(method: str, path: str):
    method = method.lower()
    segs = ['create' if method == 'post' else 'edit' if method == 'put' else method]
    path = path.replace('{', 'by-').replace('}', '').replace('-', '/').replace('_', '/').replace('/', ' ')
    segs.extend(x.lower() for x in path.split())
    return word_to_style('_'.join(segs), WordStyle.lower_camel)


def render_java_postkind_interface(method: str, path: str, title: str, vo_class, dto_class, req_query: list):
    method = method.capitalize()
    interface_name = make_interface_name(method, path)
    title = title.replace('"', "'")
    vo_varname = word_to_style(vo_class, WordStyle.lower_camel)
    dto_varname = word_to_style(dto_class, WordStyle.lower_camel)
    interface_query_body = render_interface_query_body(path, req_query)
    code_body = f"""
    @Operation(summary = "{title}", description = "")
    @{method}Mapping("{path}")
    public Result<{dto_class}> {interface_name}({interface_query_body}@RequestBody @Valid {vo_class} {vo_varname}) {{
        // todo 工具生成，需要实现
        {dto_class} {dto_varname} = null;
        return Result.success({dto_varname});
    }}
"""
    return code_body


def render_java_pagekind_interface(method: str, path: str, title: str, dto_class, req_query: list):
    method = method.capitalize()
    interface_name = make_interface_name(method, path)
    title = title.replace('"', "'")
    dto_varname = word_to_style(dto_class, WordStyle.lower_camel)
    interface_query_body = render_interface_query_body(path, req_query)
    code_body = f"""
    @Operation(summary = "{title}", description = "")
    @{method}Mapping("{path}")
    public Result<PageBean<{dto_class}>> {interface_name}({interface_query_body},
        @Valid @ParameterObject PageReq pageReq
    ) {{
        // todo 工具生成，需要实现
        IPage<{dto_class}> {dto_varname}IPage = null;
        return Result.success(PageBean.of({dto_varname}IPage));
    }}
    """
    return code_body


def render_java_getkind_interface(method: str, path: str, title: str, dto_class, req_query: list):
    method = method.capitalize()
    interface_name = make_interface_name(method, path)
    title = title.replace('"', "'")
    dto_varname = word_to_style(dto_class, WordStyle.lower_camel)
    interface_query_body = render_interface_query_body(path, req_query)
    code_body = f"""
    @Operation(summary = "{title}", description = "")
    @{method}Mapping("{path}")
    public Result<{dto_class}> {interface_name}({interface_query_body}) {{
        // todo 工具生成，需要实现
        {dto_class} {dto_varname} = null;
        return Result.success({dto_varname});
    }}
    """
    return code_body


def render_java_vo_frame(vo_package, vo_class, title, vo_body):
    code_body = f"""package {vo_package};

import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.experimental.Accessors;

import javax.validation.constraints.NotNull;

@Data
@Accessors(chain = true)
@Schema(description = "{title}")
public class {vo_class} {{
{vo_body}
}}
"""
    return code_body


def render_java_vo_body(schema_list):
    vo_body = ''
    for var_name, java_type, comment, required in schema_list:
        if required:
            annotation = '@NotNull\n    '
        else:
            annotation = ''
        schema_body = f"""
    {annotation}@Schema(description = "{comment}")
    private {java_type} {var_name};
"""
        vo_body += schema_body
    return vo_body


def render_java_entity_frame(package_name, entity_class, entity_comment, table_name, entity_body):
    code_body = f"""package {package_name};

import com.baomidou.mybatisplus.annotation.IdType;
import com.baomidou.mybatisplus.annotation.TableField;
import com.baomidou.mybatisplus.annotation.TableId;
import com.baomidou.mybatisplus.annotation.TableName;
import com.baomidou.mybatisplus.extension.handlers.JacksonTypeHandler;
import com.fasterxml.jackson.annotation.JsonProperty;
import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import com.fasterxml.jackson.databind.ser.std.ToStringSerializer;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.Data;
import lombok.experimental.Accessors;
import org.springframework.beans.BeanUtils;

import javax.validation.constraints.NotNull;
import java.time.LocalDateTime;
import java.util.List;

@Data
@Schema(description = "{entity_comment}")
@Accessors(chain = true)
@TableName(value = "{table_name}", autoResultMap = true)
public class {entity_class} {{
{entity_body}
}}
"""
    return code_body


def render_java_entity_body(schema_list):
    newline = '\n    '
    entity_body = ''
    is_primary = True
    for col_name, var_name, java_type, comment in schema_list:
        annotation = ''
        read_only = is_primary or var_name in ['createdAt', 'updatedAt', 'createTime', 'updateTime']
        if read_only:
            annotation += f'@JsonProperty(access = JsonProperty.Access.READ_ONLY){newline}'
        else:
            annotation += f'@NotNull{newline}'
        if is_primary:
            annotation += f'@TableId(value = "{col_name}", type = IdType.ASSIGN_ID){newline}'
            is_primary = False
        if java_type == 'Long':
            annotation += f'@JsonSerialize(using = ToStringSerializer.class){newline}'
        if java_type.endswith('>'):
            annotation += f'@TableField(typeHandler = JacksonTypeHandler.class){newline}'
        schema_body = f"""
    {annotation}@Schema(description = "{comment}")
    private {java_type} {var_name};  // {col_name}
"""
        entity_body += schema_body
    return entity_body


def render_java_service_impl_body(service_package, entity_package, mapper_package, class_name):
    code_body = f"""package {service_package}.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import {entity_package}.{class_name};
import {mapper_package}.{class_name}Mapper;
import {service_package}.{class_name}Service;
import org.springframework.stereotype.Service;

@Service
public class {class_name}ServiceImpl extends ServiceImpl<{class_name}Mapper, {class_name}> implements {class_name}Service {{

}}
"""
    return code_body


def render_java_service_body(service_package, entity_package, class_name):
    code_body = f"""package {service_package};

import com.baomidou.mybatisplus.extension.service.IService;
import {entity_package}.{class_name};

public interface {class_name}Service extends IService<{class_name}> {{

}}
"""
    return code_body


def render_java_mapper_body(mapper_package, entity_package, class_name):
    code_body = f"""package {mapper_package};

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import {entity_package}.{class_name};

public interface {class_name}Mapper extends BaseMapper<{class_name}> {{

}}
"""
    return code_body


# coding: utf-8
"""
arms
~~~~~~~~
CI/CD tool of Chongqing Parsec Corp.
Setup
-----
.. code-block:: bash
    > pip install arms
    > arms -h

"""

import ast
import re
from codecs import open  # To use a consistent encoding
from os import path

from setuptools import (  # Always prefer setuptools over distutils
    find_packages, setup)
from setuptools.command.install import install

_version_re = re.compile(r'__version__\s+=\s+(.*)')
version = str(ast.literal_eval(
    _version_re.search(
        open('arms/__init__.py').read()
    ).group(1)
))
here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


class MyInstall(install):
    def run(self):
        print("-- installing... --")
        install.run(self)


setup(
    name='arms',
    version=version,
    description='CI/CD tool of Chongqing Parsec Corp.',
    long_description=long_description,
    long_description_content_type='text/markdown',  # 指定内容类型为Markdown
    url='https://pypi.python.org/pypi/arms',
    author='qorzj',
    author_email='inull@qq.com',
    license='MIT',
    platforms=['any'],
    classifiers=[
        # 在这里添加分类器
    ],
    keywords='arms armstrong chongqing',
    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=[
        'lesscli>=0.2.0',
        'rich',
        'pyperclip',
        'InquirerPy',
        'json5',
        'requests',
    ],
    cmdclass={'install': MyInstall},
    entry_points={
        'console_scripts': [
            'arms = arms.main:main'
        ],
    },
)



import json
import os
import shutil
import sys
from pathlib import Path

from lesscli import add_argument, add_subcommand, run
from rich.console import Console
from rich.panel import Panel
from rich.prompt import Prompt

from arms.designer import run_design
from arms.utils.common import dump_file_name
from arms.utils.wordstyle import WordSeed, WordStyle, replace_all, replace_dict

console = Console()


def alert_and_exit(text):
    console.print(text.replace('[', '\\['), style="bold red")
    exit(1)


def terminal_menu(options, show_func):
    """
    :param options: 选项列表
    :param show_func: 从选项获取描述的函数
    :return: 选中的选项，None表示退出
    """
    page_no = 0
    page_size = 9
    total_page = (len(options) - 1) // 9 + 1
    while True:
        opt_slide = options[page_no * page_size: (page_no + 1) * page_size]
        panel_lines = []
        for idx, opt_item in enumerate(opt_slide):
            panel_lines.append('[bold][%d][/bold] < %s' %
                               (idx + 1, show_func(opt_item)))
        panel_lines.append('')
        panel_lines.append('[bold]\\[q][/bold] < Quit.')
        if page_no > 0:
            panel_lines.append('[bold]\\[k][/bold] < Previous Page.')
        if (page_no + 1) * page_size < len(options):
            panel_lines.append('[bold]\\[j][/bold] < Next Page.')
        page_hint = '' if total_page <= 1 else '(%d/%d)' % (
            page_no + 1, total_page)
        panel = Panel.fit('\n'.join(panel_lines),
                          title="Welcome to Terminal Menu%s" % page_hint)
        console.print(panel)
        print('\nChoice: ', end='')
        try:
            os.system('/bin/stty raw')
            c = sys.stdin.read(1).lower()
            print(c)
            print('\r')
        finally:
            os.system('/bin/stty cooked')
        if c == 'q':
            return None
        elif '1' <= c <= str(len(opt_slide)):
            return opt_slide[int(c) - 1]
        elif page_no > 0 and c == 'k':
            page_no -= 1
            continue
        elif (page_no + 1) * page_size < len(options) and c == 'j':
            page_no += 1
            continue
        else:
            console.print(
                "Exception: Wrong Number, Please Try Again.\n", style="bold red")
            continue


def makedir(real_path):
    from pathlib import Path
    Path(real_path).mkdir(parents=True, exist_ok=True)


def print_version():
    """
    显示版本
    """
    from arms import __version__
    text = """
    arms version: {}

    """.format(__version__)
    print(text)


def join_dot_env_and_query(dot_env_json, index_json):
    for tpl_dot_env_key, tpl_dot_env_value in index_json.get('.env', {}).items():
        yield tpl_dot_env_value, dot_env_json.get(tpl_dot_env_key, tpl_dot_env_value)
    for name_config in index_json.get('__name__', []):
        old_proj_name = name_config['word']
        new_proj_name = Prompt.ask(
            '请输入%s' % name_config['hint'], default=old_proj_name)
        yield old_proj_name, new_proj_name


def run_process(tpl_name, is_patch=False):
    """
    项目初始化工具
    """
    # [1/7]判断本地有.git目录
    if not os.path.isdir('.git'):
        alert_and_exit(
            'Please change workdir to top! or run "git init" first.')
    # [2/7]拉取模版项目
    local_path = Path.home() / '.arms_config.json'
    if not local_path.exists():
        alert_and_exit('请先执行arms config [git_url]')
    config_json = json.loads(local_path.open(encoding='utf-8').read())
    if '__default__' in config_json:
        config_json = config_json[config_json['__default__']]
    if tpl_name not in config_json:
        alert_and_exit('模版名称不存在！')
    ret = os.system('rm -rf .arms_tpl && git clone %s .arms_tpl' %
                    config_json[tpl_name]['git_url'])
    if ret:
        exit(1)
    # [3/7]生成替换字典
    json_path = Path('.arms_tpl/.arms.json')
    if not json_path.is_file():
        alert_and_exit('No .arms.json found in source project!')
    index_json = {}
    try:
        index_json.update(json.loads(json_path.open().read()))
    except Exception as e:
        alert_and_exit('.arms.json is not valid JSON format!')
    dot_env_json = {}
    try:
        local_index_json = json.loads(open('.arms.json').read())
        dot_env_json = local_index_json['.env']
    except:
        pass
    # 提示输入全局变量
    if not dot_env_json.get('ARMS_PROJECT_NAME'):
        options = ['单体项目', '微服务项目']
        opt_choice = terminal_menu(options, lambda x: x)
        if opt_choice is None:
            exit(0)
        if opt_choice == '微服务项目':
            dot_env_json['ARMS_PROJECT_NAMESPACE'] = Prompt.ask(
                '请输入项目命名空间(英文代号)')
            if WordSeed.of(dot_env_json['ARMS_PROJECT_NAMESPACE']).word_style == WordStyle.other:
                alert_and_exit("输入的[%s]不是支持的单词或词组形式！" %
                               dot_env_json['ARMS_PROJECT_NAMESPACE'])
        dot_env_json['ARMS_PROJECT_NAME'] = Prompt.ask('请输入项目名称(英文代号)')
        if WordSeed.of(dot_env_json['ARMS_PROJECT_NAME']).word_style == WordStyle.other:
            alert_and_exit("输入的[%s]不是支持的单词或词组形式！" %
                           dot_env_json['ARMS_PROJECT_NAME'])
    # 支持层级选择
    while isinstance(index_json, dict) and '__name__' not in index_json:
        options = [{'key': key, 'value': value}
                   for key, value in index_json.items() if key != '.env']
        opt_choice = terminal_menu(options, lambda x: x['key'].strip())
        if opt_choice is None:
            exit(0)
        index_json.update(opt_choice['value'])
    if not isinstance(index_json, dict):
        alert_and_exit('.arms.json错误: 节点类型错误!')
    if '__only__' in index_json:
        if '__except__' in index_json:
            alert_and_exit('.arms.json错误: __only__和__except__不能同时定义!')
        if '__rename__' in index_json:
            alert_and_exit('.arms.json错误: __only__和__rename__不能同时定义!')
    if '__rename__' in index_json:
        if any(rule.count(':') != 1 for rule in index_json['__rename__']):
            alert_and_exit('.arms.json错误: __rename__不符合规范!')
    if not isinstance(index_json['__name__'], list):
        alert_and_exit('.arms.json错误: __name__不符合规范!')
    for name_config in index_json['__name__']:
        if not name_config.get('word'):
            alert_and_exit('.arms.json错误: __name__[]["word"]不能为空!')
        if not name_config.get('hint'):
            alert_and_exit('.arms.json错误: __name__[]["hint"]不能为空!')

    # [4/7]删除无用路径
    if index_json.get('__only__'):
        only_paths = [rule.split(':')[-1] for rule in index_json['__only__']]
        rename_rules = [rule for rule in index_json['__only__'] if ':' in rule]
    else:
        only_paths = ['.']
        rename_rules = index_json.get('__rename__', [])
    except_paths = index_json.get('__except__', [])
    tar_cmd = 'tar %s -czf ../.arms_tpl.tgz --exclude .git %s' % (
        ' '.join(f'--exclude {p}' for p in except_paths), ' '.join(only_paths))
    print(tar_cmd)
    os.system(' && '.join([
        'cd .arms_tpl',
        tar_cmd,
        'cd ..',
        'rm -rf .arms_tpl',
        'mkdir .arms_tpl',
        'cd .arms_tpl',
        'tar -zxf ../.arms_tpl.tgz',
        'rm -f ../.arms_tpl.tgz'
    ]))
    # 提示输入，开始替换
    repl_dict_all = {}
    repl_result = {}  # dict[filepath] => content
    out_abs_path = os.path.abspath('.')
    for old_proj_name, new_proj_name in join_dot_env_and_query(dot_env_json, index_json):
        if old_proj_name == new_proj_name:
            continue
        # if WordSeed.of(old_proj_name).word_style == WordStyle.other:
        #     alert_and_exit("被替换的[%s]不是支持的单词或词组形式！" % old_proj_name)
        # if WordSeed.of(new_proj_name).word_style == WordStyle.other:
        #     alert_and_exit("输入的[%s]不是支持的单词或词组形式！" % new_proj_name)
        if WordSeed.of(old_proj_name).word_style == WordStyle.other and WordSeed.of(new_proj_name).word_style == WordStyle.other:
            repl_dict = replace_dict(old_proj_name, new_proj_name)
        else:
            repl_dict = {old_proj_name: new_proj_name}
        # [5/7]文件重命名
        repl_dict_all.update(repl_dict)
        os.chdir(out_abs_path)
        os.chdir('.arms_tpl')  # 变换工作目录
        for item in rename_rules:
            to_path, from_path = item.split(':', 1)
            if Path(from_path).exists():  # 前面的重命名可能会影响后面的重命名
                os.rename(from_path, to_path)  # os.rename非常强大
        curpath = Path('.')
        for i in range(20):  # 替换路径中的项目代号，最大循环20轮
            touched = False
            renames = []
            for p in curpath.rglob('*'):
                full_path = str(p)
                new_path = replace_all(full_path, repl_dict)
                if new_path != full_path:
                    renames.append(f'{new_path}:{full_path}')
            for item in renames:
                to_path, from_path = item.split(':', 1)
                if Path(from_path).exists():  # 前面的重命名可能会影响后面的重命名
                    os.rename(from_path, to_path)  # os.rename非常强大
                    touched = True
            if not touched:  # 若一轮操作没有产生重命名则退出
                break
        # [6/7]文本替换
        for p in curpath.rglob('*'):
            if p.is_dir() or str(p).startswith(('.git/', '.idea/', 'node_modules/')):
                continue
            try:
                text = p.open().read()
                new_text = replace_all(text, repl_dict)
                repl_result[str(p.relative_to(curpath))] = new_text
                if new_text != text:
                    with p.open('w') as f:
                        f.write(new_text)
            except Exception as e:
                pass
    if is_patch:
        for path, content in repl_result.items():
            print(f'**{path}**:')
            print(f'```\n{content.strip()}\n```\n')
        os.chdir(out_abs_path)
        os.system('rm -rf .arms_tpl')  # 可能由之前chdir导致
        return
    # [7/7]git add
    if not repl_dict_all:
        alert_and_exit("无文本替换，arms init已终止")
    console.print(Panel.fit(str(repl_dict_all), title='单词替换方案'))
    os.system('tar -czvf ../.arms_tpl.tgz .')
    os.chdir(out_abs_path)  # 变换工作目录
    os.system(' && '.join([
        'rm -rf .arms_tpl',
        'tar -zxf .arms_tpl.tgz',
        'rm -f .arms_tpl.tgz'
    ]))
    os.system('git add .')
    if is_patch:
        print('---- arms patch succeed :) ----')
    else:
        print('---- arms init succeed :) ----')


@add_argument('tpl_name', help='模版名')
def run_init(tpl_name):
    """
    项目初始化工具

    请输入命令完成项目初始化：arms init [tpl_name] （可先使用arms search [keyword]搜索模版名）
    """
    try:
        run_process(tpl_name=tpl_name, is_patch=False)
    except KeyboardInterrupt:
        print()
    finally:
        os.system('rm -rf .arms_tpl')
        os.system('rm -f .arms_tpl.tgz')


@add_argument('tpl_name', help='模版名')
def run_patch(tpl_name):
    """
    项目补丁工具
    arms patch [tpl_name]
    """
    try:
        run_process(tpl_name=tpl_name, is_patch=True)
    except KeyboardInterrupt:
        print()
    finally:
        os.system('rm -rf .arms_tpl')
        os.system('rm -f .arms_tpl.tgz')


def switch_config():
    local_path = Path.home() / '.arms_config.json'
    local_git_src = json.loads(local_path.open(encoding='utf-8').read())
    assert '__url__' not in local_git_src
    options = [key for key in local_git_src.keys() if key != '__default__']
    opt_choice = terminal_menu(
        options, lambda x: '(✔)' + x if x == local_git_src['__default__'] else '   ' + x)
    if opt_choice is None:
        exit(0)
    local_git_src['__default__'] = opt_choice
    with open(local_path, 'w') as f:
        f.write(json.dumps(local_git_src, ensure_ascii=False, indent=4))
    print("切换配置成功.")


def pull_config(git_url):
    # [1/2]拉取模版源
    ret = os.system("git clone %s .arms_tpl_src" % git_url)
    if ret:
        exit(1)
    # [2/2]保存到本地目录
    local_path = Path.home() / '.arms_config.json'
    try:
        try:
            local_git_src = json.loads(
                local_path.open(encoding='utf-8').read())
        except:
            local_git_src = {}
        if '__url__' in local_git_src:
            local_git_src = {local_git_src['__url__']: local_git_src}  # 兼容老版本
        patch_git_src = json.loads(open('.arms_tpl_src/config.json').read())
        local_git_src[patch_git_src['__url__']] = patch_git_src
        local_git_src['__default__'] = patch_git_src['__url__']
        with open(local_path, 'w') as f:
            f.write(json.dumps(local_git_src, ensure_ascii=False, indent=4))
        print("更新配置成功.")
    except Exception:
        print(f"无法修改本地配置文件：{local_path}")
    finally:
        shutil.rmtree('.arms_tpl_src')


def run_update():
    """
    arms更新配置
    arms update
    """
    local_path = Path.home() / '.arms_config.json'
    try:
        config_json = json.loads(local_path.open(encoding='utf-8').read())
        if '__default__' in config_json:
            config_json = config_json[config_json['__default__']]
        git_url = config_json['__url__']
        pull_config(git_url)
    except Exception:
        print(f"无法读取本地配置文件：{local_path}，请先执行arms config [git_url]")


@add_argument('git_url', help='git源访问链接', required=False)
def run_config(git_url):
    """
    arms配置工具

    arms config [git_url]  新增git源
    arms config            切换git源
    """
    if git_url:
        pull_config(git_url)
    else:
        switch_config()


@add_argument('keyword', help='搜索关键词')
def run_search(keyword):
    """
    项目搜索工具
    arms config [keyword]
    """
    keyword = keyword.lower()
    local_path = Path.home() / '.arms_config.json'
    config_json = json.loads(local_path.open(encoding='utf-8').read())
    if '__default__' in config_json:
        config_json = config_json[config_json['__default__']]
    for name, setting in config_json.items():
        if name.startswith('__'):
            continue
        if keyword == name.lower() or keyword in setting['description'].lower() or keyword in setting.get('keywords', '').lower():
            console.print(f'[bold]{name}[/bold] - {setting["description"]}')


@add_subcommand('version', print_version)
@add_subcommand('init', run_init)
@add_subcommand('patch', run_patch)
@add_subcommand('config', run_config)
@add_subcommand('search', run_search)
@add_subcommand('update', run_update)
@add_subcommand('design', run_design)  # 生成文本并复制
def entrypoint():
    """
    armstrong
    """
    pass


def main():
    if sys.version_info.major == 2 or sys.version_info.minor < 5:
        alert_and_exit('arms已不再支持python2，请安装python3.5+')
    run(entrypoint)



import ast
import json5
import re


def is_column_sql(sql_line: str):
    """
    判断一个SQL行是否是合法的列定义SQL
    """
    return not sql_line.strip().upper().startswith(('PRIMARY ', 'KEY ', 'CONSTRAINT ', 'UNIQUE ', 'FOREIGN '))


def parse_column_sql(sql_line: str):
    """
    返回：col_name, sql_type, comment
    """
    sql_line = sql_line.strip().rstrip(',')
    if ' COMMENT ' in sql_line:
        sql_line, comment_part = sql_line.split(' COMMENT ', 1)
        comment = ast.literal_eval(comment_part)
    else:
        comment = ''
    col_name, sql_type = sql_line.split(None, 2)[:2]
    col_name = col_name.strip('`')
    return col_name, sql_type, comment


def guess_mockjs(sql_type, name, comment):
    if sql_type.startswith(('enum', 'ENUM')):
        return sql_type.replace('enum', '@pick').replace('ENUM', '@pick')
    sql_type = sql_type.lower()
    if sql_type == 'date':
        return '@date'
    elif sql_type == 'datetime':
        return '@datetime'
    elif sql_type.startswith('bigint'):
        return '@natural'
    elif sql_type.startswith('int'):
        return '@integer(0, 1000000)'
    elif sql_type.startswith('double'):
        return '@float(0, 100)'
    elif sql_type.startswith('enum'):
        return sql_type.replace('enum', '@pick')
    elif '是否' in comment:
        return '@boolean'
    elif comment.endswith(('姓名', '人', '者')):
        return '@cname'
    elif sql_type.startswith('varchar') and any(
            x in name.lower() for x in ['image', 'picture', 'img', 'icon', 'avatar', 'picurl']):
        return '@image(100x100)'
    elif sql_type.startswith('varchar') and name.lower().endswith('url'):
        return 'https://httpbin.org/get?q=@word(8)'
    elif name.lower().endswith(('phone', 'mobile')):
        return '1@integer(3000000000, 9900000000)'
    elif name.lower().endswith('ipaddress') or comment.lower().endswith('ip地址'):
        return '@ip'
    elif name.lower().endswith('address') or comment.endswith('地址'):
        return '@county@cword(2)街@integer(1,100)号@cword(4)小区'
    elif name.lower().endswith('code'):
        return '@word(16)'
    elif name.lower().endswith(('title', 'name')):
        return '@cword(6)'
    else:
        return ''


def guess_java_type(mockjs):
    if isinstance(mockjs, bool):
        return 'Boolean'
    elif isinstance(mockjs, int):
        return 'Integer'
    elif isinstance(mockjs, float):
        return 'Double'
    elif isinstance(mockjs, list):
        subtype = 'String' if not mockjs else guess_java_type(mockjs[0])
        return f'List<{subtype}>'
    elif isinstance(mockjs, dict):
        return 'JSONObject'
    elif mockjs.startswith('@boolean'):
        return 'Boolean'
    elif mockjs.startswith('@integer'):
        return 'Integer'
    elif mockjs.startswith('@natural'):
        return 'Long'
    elif mockjs.startswith('@float'):
        return 'Double'
    elif mockjs.startswith('@datetime'):
        return 'LocalDateTime'
    elif mockjs.startswith('@date'):
        return 'LocalDate'
    elif mockjs.startswith('@pick'):
        return 'String'  # 后续再优化
    else:
        return 'String'


def sql_type_guess_java_type(sql_type, comment):
    sql_type = sql_type.lower()
    if sql_type == 'date':
        return 'LocalDate'
    elif sql_type == 'datetime':
        return 'LocalDateTime'
    elif sql_type == 'json':
        return 'List<String>'  # 后续改进
    elif sql_type.startswith('bigint'):
        return 'Long'
    elif sql_type.startswith('int'):
        return 'Integer'
    elif sql_type.startswith('enum'):
        return 'String'  # 后续改进
    elif sql_type.startswith('tinyint') and '是否' in comment:
        return 'Boolean'
    else:
        return 'String'


def parse_json5(json5_content: str):
    """
    返回：(data, comment_dict)
    """
    data = json5.loads(json5_content)
    comment_dict = {}
    for line in json5_content.splitlines():
        line = line.strip()
        if '//' in line and (match_ret := re.match(r'"([\w.-]+)" *:', line)):
            key = match_ret.groups()[0]
            if key not in comment_dict:
                comment_dict[key] = line.rsplit('//', 1)[-1]
    return data, comment_dict


def judge_class_name(class_name, suffix):
    assert class_name and class_name[0].isupper() and '_' not in class_name, '类名必须是大写字母开头的驼峰命名'
    assert class_name.endswith(suffix), f'类名后缀必须是{suffix}'
